"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

from django.contrib.auth import get_user_model
from django.contrib.auth import REDIRECT_FIELD_NAME

from django.http import HttpResponse, HttpResponseRedirect
from django.views.decorators.cache import never_cache
from django_tequila.tequila_client import TequilaClient
from django_tequila.tequila_client import EPFLConfig
from django.conf import settings


User = get_user_model()


def login(request):
    if request.GET.get(REDIRECT_FIELD_NAME):
        next_path = request.GET[REDIRECT_FIELD_NAME]
    else:
        next_path = settings.LOGIN_REDIRECT_URL

    # fullfill domain for tequila, and for security reasons
    next_path = request.get_host() + next_path

    if request.is_secure():
        next_path = 'https://' + next_path
    else:
        next_path = 'http://' + next_path

    if request.user.is_authenticated():
        return HttpResponseRedirect(next_path)

    try:
        server_url = settings.TEQUILA_SERVER_URL
    except AttributeError:
        server_url = ""

    try:
        additional_params = settings.TEQUILA_CONFIG_ADDITIONAL
    except AttributeError:
        additional_params = None

    try:
        allows_needed = settings.TEQUILA_CONFIG_ALLOW
    except AttributeError:
        allows_needed = None

    try:
        service_name = settings.TEQUILA_SERVICE_NAME
    except AttributeError:
        service_name = 'Unknown application'

    try:
        strong_authentication = settings.TEQUILA_STRONG_AUTHENTICATION
    except AttributeError:
        strong_authentication = False

    tequila_client = TequilaClient(EPFLConfig(server_url = server_url,
                                        additional_params = additional_params,
                                        redirect_to = next_path,
                                        allows = allows_needed,
                                        service = service_name,
                                        allow_guests = True,
                                        strong_authentication = strong_authentication,
                                        ))

    request.session.set_test_cookie()

    return HttpResponseRedirect(tequila_client.login_url())


login = never_cache(login)


def logout(request):
    if request.GET.get(REDIRECT_FIELD_NAME):
        next_path = request.GET[REDIRECT_FIELD_NAME]
    else:
        next_path = settings.LOGOUT_URL

    from django.contrib.auth import logout as auth_logout
    auth_logout(request)

    return HttpResponseRedirect(next_path)


def not_allowed(request):
    return HttpResponse("Not allowed")



"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

from __future__ import absolute_import

from .client import TequilaClient
from .config import Config, EPFLConfig



"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

import logging

from django.conf import settings
from django.contrib import auth
from django.contrib.auth.middleware import PersistentRemoteUserMiddleware
from django.core.exceptions import ImproperlyConfigured

from django.http import HttpResponseRedirect
from django.utils.http import urlencode

logger = logging.getLogger('django_tequila.middleware')


def get_query_string(params, new_params=None, remove=None):
    """ Allow to rewrite params from url """
    if new_params is None:
        new_params = {}
    if remove is None:
        remove = []
    p = params.copy()
    for r in remove:
        for k in list(p.keys()):
            if k.startswith(r):
                del p[k]

    for k, v in new_params.items():
        if v is None:
            if k in p:
                del p[k]
        else:
            p[k] = v

    for k, v in p.items():
        if isinstance(v, (list, tuple)):
            p[k] = v[0]

    return '?%s' % urlencode(p)


class TequilaMiddleware(PersistentRemoteUserMiddleware):
    """
    Middleware for utilizing tequila web-server-provided authentication.

    If request.user is not authenticated, then this middleware attempts to
    authenticate with the key passed in the ``key`` request.GET.
    If authentication is successful, the user is automatically logged in to
    persist the user in the session.
    """

    # Name of request key to grab the key from
    header = "key"

    def process_request(self, request):
        # AuthenticationMiddleware is required so that request.user exists.
        if not hasattr(request, 'user'):
            raise ImproperlyConfigured(
                "The Django remote user auth middleware requires the"
                " authentication middleware to be installed.  Edit your"
                " MIDDLEWARE setting to insert"
                " 'django.contrib.auth.middleware.AuthenticationMiddleware'"
                " before the TequilaMiddleware class.")
        # If the user is already authenticated, her is already
        # persisted in the session and we don't need to continue.
        if request.user.is_authenticated():
            return

        try:
            # Django diff : key is provided by GET, not META
            tequila_key = request.GET[self.header]
        except KeyError:
            # If specified header doesn't exist then remove any existing
            # authenticated remote-user, or return (leaving request.user set to
            # AnonymousUser by the AuthenticationMiddleware).
            if self.force_logout_if_no_header and request.user.is_authenticated:
                self._remove_invalid_user(request)
            return

        # We are seeing this user for the first time in this session, attempt
        # to authenticate the user.
        logger.debug("First time user found, going for authentication "
                     "with the key %s..." % tequila_key)
        user = auth.authenticate(tequila_key=tequila_key)

        logger.debug("User found and logged : %s" % user.__dict__)

        if hasattr(user, 'profile'):
            logger.debug("User profile : %s" % user.profile.__dict__)
        else:
            logger.debug("The current appplication has no profile model "
                         "set, returned informations will not be saved")

        # deny page if not allowed
        if not user:
            logger.debug("User was not able to be authenticated, "
                         "redirect to the not allowed page")
            return HttpResponseRedirect(settings.LOGIN_REDIRECT_IF_NOT_ALLOWED)
        else:
            # User is valid.  Set request.user and persist user in the session
            # by logging the user in.
            request.user = user
            auth.login(request, user)
            logger.debug("User found and logged : %s" % user)

            try:
                clean_url = settings.TEQUILA_CLEAN_URL

                if clean_url:
                    # get the url, remove key and redirect to it
                    cleaned_url = request.path

                    # QueryDict to dict
                    params = request.GET.dict()

                    cleaned_url += get_query_string(params, remove=[self.header])
                    return HttpResponseRedirect(cleaned_url)

            except AttributeError:
                pass



"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.shortcuts import render
from django.utils.safestring import mark_safe


def index(request):
    user_info = request.user.__dict__

    if request.user.is_authenticated:
        user_info.update(request.user.profile.__dict__)

    return render(request, 'index.html', {
        'user': request.user,
        'user_info': user_info,
    })


@login_required
def protected_view(request):
    profile = ""

    if request.user.is_authenticated():
        profile = request.user.profile

    return HttpResponse("Successfully seeing a protected view.")


def unprotected_view(request):
    login_url = mark_safe('<a href="%s?next=%s">login url</a>' % (
    reverse('login_view'), request.path))
    logout_url = mark_safe('<a href="%s?next=%s">logout url</a>' % (
    reverse('logout'), request.path))

    context = {'user': request.user,
               'logout_url': logout_url}

    return render(request, 'unprotected_view.html', context)



"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

from setuptools import setup

setup(
    name='django-tequila',
    version=__import__('django_tequila').__version__,

    author='Julien Delasoie',
    author_email='julien.delasoie@epfl.ch',
    keywords='django, tequila, authentication',

    url='https://github.com/epfl-idevelop/django-tequila',
    license="LGPLv3",
    description='A Tequila authentication backend for django',
    long_description=open('README.rst', 'r').read(),

    packages=['django_tequila',
              'django_tequila.django_backend',
              'django_tequila.tequila_client',
              'django_tequila.tequila_auth_views'],

    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: System :: Systems Administration :: Authentication/Directory',
    ]
)



"""
    (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, VPSI, 2017
"""

from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import User

from sample_app.models import UserProfile


class UserProfileInline(admin.StackedInline):
    model = UserProfile


class CustomUserAdmin(UserAdmin):
    inlines = [UserProfileInline]


admin.site.unregister(User)
admin.site.register(User, CustomUserAdmin)
